package bicliqueviz;

import javax.swing.JFrame;
import javax.swing.JSplitPane;
import javax.swing.JScrollPane;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import java.io.File;
import java.util.Vector;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;
import java.io.BufferedWriter;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.FileReader;
import java.io.IOException;
import java.awt.event.KeyEvent;
import javax.swing.KeyStroke;

import javax.swing.JOptionPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.JList;
import javax.swing.tree.TreePath;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionListener;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import javax.swing.text.Position;

import edu.uci.ics.jung.graph.UndirectedSparseGraph;

import java.awt.event.ActionEvent;
import java.awt.GridLayout;
import edu.uci.ics.screencap.Dump;
import edu.uci.ics.screencap.EPSDump;
import com.centerkey.utils.BareBonesBrowserLaunch;

/**
 * 
 * This class represents the window of the application
 * 
 * @author Martin Schwartz
 *
 */
public class BiCliqueWin extends JFrame implements ActionListener, ChangeListener
{
	private final static long serialVersionUID = 5;
	private BiGraph graph;
	private JSplitPane splitPane;
	private JScrollPane westPanel;
	private JScrollPane topPanel;
	private JScrollPane bottomPanel;
	private JScrollPane middlePanel;
	private JScrollPane commPanel;
	private Dimension screenSize;
	private JMenuItem open;
	private JMenuItem openMinusLCM;
	private JMenuItem exportComm;
	private JMenuItem exportAllComm;
	private JMenuItem extractAllComm;
	private JMenuItem screenshot;
	private JMenuItem exit;
	private JMenuItem vertexComm;
	private JMenuItem clearSelection;
	private JMenuItem nodeSearch;
	private JMenuItem commSearch;
	private JMenuItem communityNetwork;
	private JMenuItem biHistogram;
	private JMenuItem degHistogram;
	private JMenuItem commHistogram;
	private JMenuItem about;
	private JMenuItem help;
	private JSpinner sizeSpinner;
	private JSpinner labelSizeSpinner;
	private JCheckBoxMenuItem scaling;
	private JButton sortButton;
	private TabbedPane upperTabs;
	private TabbedPane lowerTabs;
	private BicliqueTreeMenu treeMenu;
	private final JFileChooser fc = new JFileChooser(System.getProperty("user.dir"));
	private InfoFile infoFile;
	private VertexCommList vList;
	public static final int WARNING_NODESIZE = 100;
	public static int[][] histogramMatrix;
	public static int maxHistogramValue;
	public final static int MAX_HISTOGRAM_SIZE = 15;
	public static String size_file = null;
	public static String select_file = null;

	/**
	 * Create new window
	 */
	public BiCliqueWin() 
	{
		super();
		init();
	}
	
	private void init()
	{
		setExtendedState(Frame.MAXIMIZED_BOTH);
		setLayout(new BorderLayout());
		setTitle("BCFinder");
		this.setJMenuBar(createMenuBar());
		screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension d = screenSize;
		d.setSize(d.getWidth()/2.0, d.getHeight()/2.0);
		setSize(d);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setLocationRelativeTo(null);
		splitPane = initSplitPane();
		this.getContentPane().add(splitPane);
		
		histogramMatrix = new int[MAX_HISTOGRAM_SIZE][MAX_HISTOGRAM_SIZE];
		
		for(int i = 0; i < MAX_HISTOGRAM_SIZE; i++)
		{
			for(int j = 0; j < MAX_HISTOGRAM_SIZE; j++)
			{
				histogramMatrix[i][j] = 0;
			}
		}
		maxHistogramValue = 0;
		
		setVisible(true);
	}
	
	private JMenuBar createMenuBar()
	{
		JMenuBar menuBar = new JMenuBar();
		
		JMenu fileMenu = new JMenu("File");
		
		String osName = System.getProperty("os.name");
		
		int mask_a = ActionEvent.CTRL_MASK;
		int mask_b = ActionEvent.SHIFT_MASK;
		int mask_c = ActionEvent.ALT_MASK;
		
		if (osName.startsWith("Mac OS"))
		{
			mask_a = ActionEvent.META_MASK;
		}
		
		fileMenu.setMnemonic(KeyEvent.VK_F);
		
		
		open = new JMenuItem("Open");
		open.addActionListener(this);
		open.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_O, mask_a));
		fileMenu.add(open);
		
		openMinusLCM = new JMenuItem("Open (LCM already done)");
		openMinusLCM.addActionListener(this);
		openMinusLCM.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_O, mask_a | mask_b));
		fileMenu.add(openMinusLCM);
		
		exportComm = new JMenuItem("Export Community");
		exportComm.addActionListener(this);
		exportComm.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_E, mask_a));
		fileMenu.addSeparator();
		fileMenu.add(exportComm);
		
		exportAllComm = new JMenuItem("Export All Communities");
		exportAllComm.addActionListener(this);
		exportAllComm.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_E, mask_a | mask_b ));
		fileMenu.add(exportAllComm);
		
		extractAllComm = new JMenuItem("Extract All Communities");
		extractAllComm.addActionListener(this);
		extractAllComm.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_E, mask_a | mask_b | mask_c ));
		fileMenu.add(extractAllComm);
		
		screenshot = new JMenuItem("Save community as eps...");
		screenshot.addActionListener(this);
		screenshot.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_P, mask_a ));
		fileMenu.addSeparator();
		fileMenu.add(screenshot);
		
		exit = new JMenuItem("Exit");
		exit.addActionListener(this);
		exit.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_Q, mask_a ));
		fileMenu.addSeparator();
		fileMenu.add(exit);
		
		menuBar.add(fileMenu);
		
		JMenu toolsMenu = new JMenu("Tools");
		toolsMenu.setMnemonic(KeyEvent.VK_T);
		
		vertexComm = new JMenuItem("Show Communities for selected vertices");
		vertexComm.addActionListener(this);
		toolsMenu.add(vertexComm);
		
		clearSelection = new JMenuItem("Clear Selection");
		clearSelection.addActionListener(this);
		clearSelection.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_D, mask_a ));
		toolsMenu.add(clearSelection);
		
		scaling = new JCheckBoxMenuItem("Vertex scaling", false);
		scaling.addActionListener(this);
		scaling.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_S, mask_b ));
		toolsMenu.add(scaling);
		
		nodeSearch = new JMenuItem("Search for node...");
		nodeSearch.addActionListener(this);
		nodeSearch.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_F, mask_a ));
		toolsMenu.add(nodeSearch);
		
		commSearch = new JMenuItem("Search for community...");
		commSearch.addActionListener(this);
		commSearch.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_F, mask_a | mask_b ));
		toolsMenu.add(commSearch);
		
		communityNetwork = new JMenuItem("Community Network");
		communityNetwork.addActionListener(this);
		communityNetwork.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_G, mask_a ));
		toolsMenu.add(communityNetwork);
		
		biHistogram = new JMenuItem("Biclique histogram");
		biHistogram.addActionListener(this);
		biHistogram.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_H, mask_a | mask_b ));
		toolsMenu.add(biHistogram);
		
		degHistogram = new JMenuItem("Node degree histogram");
		degHistogram.addActionListener(this);
		degHistogram.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_H, mask_a | mask_b | mask_c ));
		toolsMenu.add(degHistogram);
		
		commHistogram = new JMenuItem("Comm. size histogram");
		commHistogram.addActionListener(this);
		commHistogram.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_H, mask_a | mask_b | mask_c ));
		toolsMenu.add(commHistogram);
		
		toolsMenu.addSeparator();
		JLabel sizeLabel = new JLabel("Set node size:");
		sizeSpinner = new JSpinner(new SpinnerNumberModel(3,1,100,1));
		sizeSpinner.addChangeListener(this);
		sizeLabel.setLabelFor(sizeSpinner);
		toolsMenu.add(sizeLabel);
		toolsMenu.add(sizeSpinner);
		
		JLabel labelSizeLabel = new JLabel("Set label size:");
		labelSizeSpinner = new JSpinner(new SpinnerNumberModel(12,1,100,1));
		labelSizeSpinner.addChangeListener(this);
		sizeLabel.setLabelFor(labelSizeSpinner);
		toolsMenu.add(labelSizeLabel);
		toolsMenu.add(labelSizeSpinner);
		
		menuBar.add(toolsMenu);
		
		JMenu helpMenu = new JMenu("Help");
		helpMenu.setMnemonic(KeyEvent.VK_H);
		
		help = new JMenuItem("Help...");
		help.addActionListener(this);
		helpMenu.add(help);
		
		about = new JMenuItem("About BCFinder");
		about.addActionListener(this);
		helpMenu.add(about);
		
		menuBar.add(helpMenu);
		
		return menuBar;
	}
	
	/**
	 * @return the middle panel (the one containing the visualisation of the graph)
	 */
	public JScrollPane getMiddlePanel()
	{
		return middlePanel;
	}
	
	private JSplitPane initSplitPane()
	{
		topPanel = new JScrollPane();
		topPanel.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
		upperTabs = new TabbedPane(this);
		lowerTabs = new TabbedPane(this);
		upperTabs.setOtherTabbedPane(lowerTabs);
		lowerTabs.setOtherTabbedPane(upperTabs);
		topPanel.setViewportView(upperTabs.getTabbedPane());
		middlePanel = new JScrollPane();
		bottomPanel = new JScrollPane();
		bottomPanel.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
		bottomPanel.setViewportView(lowerTabs.getTabbedPane());
		JSplitPane pane3 = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
				middlePanel, bottomPanel);
		JSplitPane pane1 = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
				topPanel, pane3);
		pane1.setResizeWeight(0.25);
		pane3.setResizeWeight(0.65);
		westPanel = new JScrollPane();
		JSplitPane pane2 = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                westPanel, pane1);
		pane2.setResizeWeight(0.15);
		pane2.setOneTouchExpandable(true);
		return pane2;
	}
	
	private void openFile(ProgWin pw, boolean withLCM)
	{
        fc.setDialogTitle("Choose bigraph file");
		int returnVal = fc.showOpenDialog(this);
		
		long t1 = 0;
		
		if (returnVal == JFileChooser.APPROVE_OPTION)
		{
			String fileName = fc.getSelectedFile().getPath();
			String idFileA = null;
			String idFileB = null;
			
			File cacheDir = new File(fileName+"_files");
			
			if(!cacheDir.exists())
			{
				if(JOptionPane.showOptionDialog(this, "Do you want to specify Node Info files?", 
						"Choose Node Info files?", JOptionPane.YES_NO_OPTION, 
						JOptionPane.QUESTION_MESSAGE, 
						null, null, null) == JOptionPane.YES_OPTION)
				{
					fc.setDialogTitle("Choose vertex A Info file");
					returnVal = fc.showOpenDialog(this);
					
					if (returnVal == JFileChooser.APPROVE_OPTION)
					{
						idFileA = fc.getSelectedFile().getPath();
						fc.setDialogTitle("Choose vertex B Info file");
						returnVal = fc.showOpenDialog(this);
						
						if (returnVal == JFileChooser.APPROVE_OPTION)
						{
							idFileB = fc.getSelectedFile().getPath();
						}
						else
						{
							idFileA = null;
						}
					}
				}
			}
			String minThresA = null;
			String minThresB = null;
			String maxThresA = null;
			String maxThresB = null;
			
			boolean fastMethod = true;
			
			if(!cacheDir.exists())
			{
				while(minThresA == null)
				{
					minThresA = JOptionPane.showInputDialog(this, "Pick min. threshold for A: (must be greater than 0)",
							"Min. threshold", JOptionPane.QUESTION_MESSAGE);
					
					try 
					{
						int a = Integer.parseInt(minThresA);
						
						if(a <= 0)
						{
							minThresA = null;
						}
					} 
					catch (NumberFormatException e) 
					{
						minThresA = null;
					}
				}
				
				while(minThresB == null)
				{
					minThresB = JOptionPane.showInputDialog(this, "Pick min. threshold for B: (must be greater than 0)",
							"Min. threshold", JOptionPane.QUESTION_MESSAGE);
					
					try 
					{
						int b = Integer.parseInt(minThresB);
						
						if(b <= 0)
						{
							minThresB = null;
						}
					} 
					catch (NumberFormatException e) 
					{
						minThresB = null;
					}
				}
				
				while(maxThresA == null)
				{
					maxThresA = JOptionPane.showInputDialog(this, "Pick max. threshold for A: (must be greater than \nmin. threshold for A, or -1 for none)",
							"Max. threshold", JOptionPane.QUESTION_MESSAGE);
					
					try 
					{
						int a = Integer.parseInt(maxThresA);
						
						if(a == -1)
						{
							a = Integer.MAX_VALUE;
							maxThresA = String.valueOf(a);
						}
						else if(a <= Integer.parseInt(minThresA))
						{
							maxThresA = null;
						}
					} 
					catch (NumberFormatException e) 
					{
						maxThresA = null;
					}
				}
				
				while(maxThresB == null)
				{
					maxThresB = JOptionPane.showInputDialog(this, "Pick max. threshold for B: (must be greater than \nmin. threshold for B, or -1 for none)",
							"Max. threshold", JOptionPane.QUESTION_MESSAGE);
					
					try 
					{
						int b = Integer.parseInt(maxThresB);
						
						if(b == -1)
						{
							b = Integer.MAX_VALUE;
							maxThresB = String.valueOf(b);
						}
						else if(b <= Integer.parseInt(minThresB))
						{
							maxThresB = null;
						}
					} 
					catch (NumberFormatException e) 
					{
						maxThresB = null;
					}
				}
				
				/*int answer = JOptionPane.showConfirmDialog(null, "Do you want to use fast loading?\n" +
						"If you want to load a large network choose NO\n" +
						"as the program will run out of memory else choose YES", "Fast loading", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
				
				if(answer == JOptionPane.NO_OPTION)
				{
					fastMethod = false;
				}*/
			}
			
			t1 = System.currentTimeMillis();
			pw.showWin(true);
			
			graph = new BiGraph(this, middlePanel.getSize());
			
			String name = fileName;
			int index = name.lastIndexOf(".");
			if(index > 0)
			{
				name = name.substring(0, index);
			}
			String size = name+".sz";
			
			size_file = size;
			select_file = fileName;
			
			if(cacheDir.exists())
			{
				infoFile = new InfoFile();
				CacheFilesLoader loader = new CacheFilesLoader(cacheDir.getPath(), pw);
				
				loader.init(this);
				vList = new VertexCommList(graph, this, upperTabs, lowerTabs);
				treeMenu = new BicliqueTreeMenu(loader, 
						graph, upperTabs, lowerTabs, vList, this);
				
				upperTabs.setTreeMenu(treeMenu);
				lowerTabs.setTreeMenu(treeMenu);
				upperTabs.setVertexCommList(vList);
				lowerTabs.setVertexCommList(vList);
				
				JPanel west = new JPanel(new GridLayout(2,0,0,10));
				JScrollPane jsp1 = new JScrollPane();
				jsp1.setViewportView(treeMenu.getJTree());
				west.add(jsp1);
				
				commPanel = new JScrollPane();
				JPanel jp = new JPanel(new BorderLayout());
				sortButton = new JButton("Sort by B");
				sortButton.addActionListener(this);
				jp.add(sortButton, BorderLayout.NORTH);
				jp.add(vList.getJList(), BorderLayout.CENTER);
				commPanel.setViewportView(jp);
				west.add(commPanel);
				
				westPanel.setViewportView(west);
				westPanel.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
				westPanel.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
				middlePanel.setViewportView(graph.getZoomPane());
				graph.getVV().setVisible(false);
			}
			else
			{	
				String bigraph = name+".bi";
				
				if(withLCM)
				{
					LCM.runLCM(pw, fileName, bigraph, size, Integer.parseInt(minThresA), Integer.parseInt(minThresB), 
							Integer.parseInt(maxThresA), Integer.parseInt(maxThresB), true);
				}
				
				vList = new VertexCommList(graph, this, upperTabs, lowerTabs);
				treeMenu = new BicliqueTreeMenu(size, bigraph, 
						graph, upperTabs, lowerTabs, vList, this, 
						pw, Integer.parseInt(minThresA), Integer.parseInt(minThresB),
						Integer.parseInt(maxThresA), Integer.parseInt(maxThresB));
				
				if(fastMethod)
				{
					treeMenu.createOverlapMatrix2(pw);
				}
				else
				{
					treeMenu.createOverlapMatrix(pw);
				}
				infoFile = new InfoFile(idFileA, idFileB, pw);
				treeMenu.createTree(pw,infoFile);
				upperTabs.setTreeMenu(treeMenu);
				lowerTabs.setTreeMenu(treeMenu);
				upperTabs.setVertexCommList(vList);
				lowerTabs.setVertexCommList(vList);
				
				JPanel west = new JPanel(new GridLayout(2,0,0,10));
				JScrollPane jsp1 = new JScrollPane();
				jsp1.setViewportView(treeMenu.getJTree());
				west.add(jsp1);
				
				commPanel = new JScrollPane();
				JPanel jp = new JPanel(new BorderLayout());
				sortButton = new JButton("Sort by B");
				sortButton.addActionListener(this);
				jp.add(sortButton, BorderLayout.NORTH);
				jp.add(vList.getJList(), BorderLayout.CENTER);
				commPanel.setViewportView(jp);
				west.add(commPanel);
				
				westPanel.setViewportView(west);
				westPanel.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
				westPanel.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
				middlePanel.setViewportView(graph.getZoomPane());
				graph.getVV().setVisible(false);
				
				Vector<String> children = new Vector<String>();
				
				for(int i = 0; i < treeMenu.getRoot().getChildCount(); i++)
				{
					children.add(treeMenu.getRoot().getChildAt(i).toString());
				}
				
				CacheFilesSaver cfm = new CacheFilesSaver(fileName, children, treeMenu.getCommunitiesTree(), treeMenu.getBicliques(), infoFile);
				cfm.createCacheFiles();
			}
			
			createHistogramMatrix(size);
		}
		pw.dispose();
		long t2 = System.currentTimeMillis();
		
		if(treeMenu != null)
		{
			System.out.println();
			System.out.println("Time elapsed: "+ (t2-t1));
		}
	}
	
	private void createHistogramMatrix(String sz)
	{
		try 
		{
			BufferedReader reader = new BufferedReader(new FileReader(sz));
			
			String line = reader.readLine();
			
			while(line != null)
			{
				String[] splitLine = line.split("\\s");
				int a = Integer.parseInt(splitLine[0]);
				int b = Integer.parseInt(splitLine[1]);
				
				if(a > MAX_HISTOGRAM_SIZE)
				{
					a = MAX_HISTOGRAM_SIZE;
				}
				if(b > MAX_HISTOGRAM_SIZE)
				{
					b = MAX_HISTOGRAM_SIZE;
				}
				histogramMatrix[a-1][b-1]++;
				
				maxHistogramValue = Math.max(maxHistogramValue, histogramMatrix[a-1][b-1]);
				line = reader.readLine();
			}
		} 
		catch (FileNotFoundException e) 
		{
			e.printStackTrace();
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * @param jl Jlist for communities panel
	 */
	public void setCommPanel(JList jl)
	{
		JPanel jp = new JPanel(new BorderLayout());
		jp.add(sortButton, BorderLayout.NORTH);
		jp.add(jl, BorderLayout.CENTER);
		commPanel.setViewportView(jp);
	}
	
	/**
	 * @return ID files instance
	 */
	public InfoFile getInfoFile()
	{
		return infoFile;
	}
	
	/**
	 * @return Tree menu instance
	 */
	public BicliqueTreeMenu getTreeMenu()
	{
		return treeMenu;
	}
	
	/**
	 * Clear selection of the graph
	 */
	public void clearGraphSelection()
	{
		graph.getVV().getPickedVertexState().clear();
    	graph.getVV().getPickedEdgeState().clear();
	}
	
	/**
	 * Clear selection of all lists in the tabbed panes (biclique lists) and the communities list
	 */
	public void clearListSelection()
	{
    	upperTabs.getEdgesList().clearSelection();
    	upperTabs.getVertexList().clearSelection();
    	
    	lowerTabs.getEdgesList().clearSelection();
    	lowerTabs.getVertexList().clearSelection();
	}
	
	/**
	 * Clears the vertex community list
	 */
	public void clearVertexCommList()
	{
		vList.clearList();
	}
	
	/**
	 * Clear selection of biclique lists in the tabbed panes
	 */
	public void clearBicliqueList()
	{
		if(upperTabs.getBicliquesList() != null)
		{
			upperTabs.getBicliquesList().clearSelection();
			lowerTabs.getBicliquesList().clearSelection();
		}
	}
	
	private Vector<VertexObj> searchForNode(String s)
	{
		Vector<VertexObj> vv = new Vector<VertexObj>();
		
		if(s.length() > 0)
		{
			Iterator<VertexObj> it = infoFile.getIDToInfo().values().iterator();
			
			while(it.hasNext())
			{
				VertexObj v = it.next();
				
				String name = v.getName().toLowerCase();
				s = s.toLowerCase();
				
				if(name.matches(".*"+s+".*"))
				{
					vv.add(v);
				}
			}
		}
		
		return vv;
	}
	
	private Vector<Community> searchForCommunity(String s)
	{
		Vector<Community> vv = new Vector<Community>();
		
		if(s.length() > 0)
		{
			Vector<TreeSet<Community>> comms = treeMenu.getCommunitiesTree();
			
			for(int i = 0; i < comms.size(); i++)
			{
				Iterator<Community> it = comms.get(i).iterator();
				
				while(it.hasNext())
				{
					Community c = it.next();
					
					String id = String.valueOf(c.getID());
					s = s.toLowerCase();
					
					if(id.matches(s+".*"))
					{
						vv.add(c);
					}
				}
			}
		}
		
		return vv;
	}
	
	/**
	 * @param cc
	 */
	public void selectCommunity(Community cc)
	{
		Vector<TreeSet<Community>> comms = treeMenu.getCommunitiesTree();
		
		for(int i = 0; i < comms.size(); i++)
		{
			Iterator<Community> it = comms.get(i).iterator();
			
			while(it.hasNext())
			{
				Community c = it.next();
				
				if(c.toString().equals(cc.toString()))
				{
					treeMenu.getJTree().expandRow(i+1);
					TreePath tp = treeMenu.getJTree().getNextMatch(cc.toString(), i+1, Position.Bias.Forward);
					treeMenu.getJTree().scrollPathToVisible(tp);
					treeMenu.getJTree().setSelectionPath(tp);
				}
			}
		}
	}
	
	/**
	 * Sets the vertex community list for the selected vertices
	 * @param set the set of vertices to find communities for
	 */
	public void setVlist(Set<VertexObj> set)
	{
		vList.setJList(graph.findCommunities(set));
		JPanel jp = new JPanel(new BorderLayout());
		jp.add(sortButton, BorderLayout.NORTH);
		jp.add(vList.getJList(), BorderLayout.CENTER);
		commPanel.setViewportView(jp);
	}
	
	/**
	 * Export the selected community to three seperate files
	 * one for node A names/IDs, one for Node B names/IDS and
	 * one for the links between them
	 */
	public void exportCommunity(UndirectedSparseGraph<VertexObj, EdgeObj> g)
	{
		if(g != null)
		{
			if(g.getVertexCount() > 0)
			{
				fc.setDialogTitle("Export Community");
				int returnVal = fc.showSaveDialog(this);
				
				if (returnVal == JFileChooser.APPROVE_OPTION)
				{
					createExportFile(fc.getSelectedFile(), null, g);
				}
			}
		}
	}
	
	private void createExportFile(File selectedFile, Integer index, UndirectedSparseGraph<VertexObj, EdgeObj> g)
	{
		try 
		{
			String parentPath = selectedFile.getParent()+"/";
			String fName = selectedFile.getName();
			
			if(index != null)
			{
				fName = index+"-"+selectedFile.getName();
			}
			
			if(parentPath != null)
			{
				fName = parentPath+fName;
			}
			
			String aFile = selectedFile.getName();
			
			if(index == null)
			{
				aFile = "a-"+selectedFile.getName();
			}
			else
			{
				aFile = "a-"+index+"-"+selectedFile.getName();
			}
			
			if(parentPath != null)
			{
				aFile = parentPath+aFile;
			}
			
			BufferedWriter nodeAWriter = new BufferedWriter(new FileWriter(aFile));
			
			String bFile = selectedFile.getName();
			
			if(index == null)
			{
				bFile = "b-"+selectedFile.getName();
			}
			else
			{
				bFile = "b-"+index+"-"+selectedFile.getName();
			}
			
			if(parentPath != null)
			{
				bFile = parentPath+bFile;
			}
			
			BufferedWriter nodeBWriter = new BufferedWriter(new FileWriter(bFile));
			
			TreeSet<VertexObj> vs = new TreeSet<VertexObj>(new VertexObjComparator());
			vs.addAll(g.getVertices());
			
			Iterator<VertexObj> it = vs.iterator();
			
			while(it.hasNext())
			{
				VertexObj v = it.next();
				
				if(v.isVertexA())
				{
					nodeAWriter.write(v.getID()+"\t"+v.getName()+"\n");
				}
				else
				{
					nodeBWriter.write(v.getID()+"\t"+v.getName()+"\n");
				}
			}
			
			nodeAWriter.close();
			nodeBWriter.close();
			
			Iterator<EdgeObj> it2 = g.getEdges().iterator();
			
			TreeSet<NumberPair> es = new TreeSet<NumberPair>(new NumberPairComparer());
			
			while(it2.hasNext())
			{
				EdgeObj e = it2.next();
				
				NumberPair np = new NumberPair(g.getEndpoints(e).getFirst().getID(), 
						g.getEndpoints(e).getSecond().getID());
				es.add(np);
			}
			
			BufferedWriter writer = new BufferedWriter(new FileWriter(fName));
			
			Iterator<NumberPair> it3 = es.iterator();
			
			while(it3.hasNext())
			{
				NumberPair np = it3.next();
				
				writer.write(np.getA()+"\t"+np.getB()+"\n");
			}
			
			writer.close();
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
	
	public void exportAllCommunities()
	{
		fc.setDialogTitle("Export Communities");
		int returnVal = fc.showSaveDialog(this);
		
		if (returnVal == JFileChooser.APPROVE_OPTION)
		{
			for(int i = 0; i < treeMenu.getCommunitiesTree().size(); i++)
			{
				TreeSet<Community> cs = treeMenu.getCommunitiesTree().get(i);
				
				Iterator<Community> it = cs.iterator();
				
				while(it.hasNext())
				{
					Community comm = it.next();
					
					createExportFile(fc.getSelectedFile(), comm.getID(), graph.createBiclique(comm.getBicliques()));
				}
			}
			
		}
	}
	
	private void createExtractFile(File selectedFile, String index, Community c)
	{
		try 
		{
			String parentPath = selectedFile.getParent()+"/";
			String fName = selectedFile.getName();
			
			if(index != null)
			{
				fName = index+"-"+selectedFile.getName();
			}
			
			if(parentPath != null)
			{
				fName = parentPath+fName;
			}
			
			BufferedWriter writer = new BufferedWriter(new FileWriter(fName, true));
			Iterator<Integer> it = c.getAllUpperNodes().iterator();
			
			writer.write(c.getID()+"a:\t");
			while(it.hasNext())
			{
				Integer id = it.next();
				writer.write(id+"\t");
			}
			writer.write("\n");
			
			it = c.getAllLowerNodes().iterator();
			
			writer.write(c.getID()+"b:\t");
			while(it.hasNext())
			{
				Integer id = it.next();
				writer.write(id+"\t");
			}
			writer.write("\n");
			
			writer.close();
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
	
	public void extractAllCommunities()
	{
		fc.setDialogTitle("Extract Communities");
		int returnVal = fc.showSaveDialog(this);
		
		if (returnVal == JFileChooser.APPROVE_OPTION)
		{
			for(int i = 0; i < treeMenu.getCommunitiesTree().size(); i++)
			{
				TreeSet<Community> cs = treeMenu.getCommunitiesTree().get(i);
				
				Iterator<Community> it = cs.iterator();
				
				while(it.hasNext())
				{
					Community comm = it.next();
					
					createExtractFile(fc.getSelectedFile(), treeMenu.getRoot().getChildAt(i).toString(), comm);
				}
			}
			
		}
	}
	
	public void takeScreenshot(Component c)
	{
		Dump dumper = new EPSDump();
		fc.setDialogTitle("Screenshot");
		int returnVal = fc.showSaveDialog(this);
		
		if (returnVal == JFileChooser.APPROVE_OPTION)
		{
			try 
			{
				File f = new File(fc.getSelectedFile().getPath()+".eps");
				dumper.dumpComponent(f, c);
			} 
			catch (IOException e) 
			{
				e.printStackTrace();
			}
		}
	}
	
	public void actionPerformed(ActionEvent e) 
	{
		Object o = e.getSource();
        if(o.equals(open))
        {
        	ProgWin pw = new ProgWin();
        	openFile(pw, true);
        }
        else if(o.equals(openMinusLCM))
        {
        	ProgWin pw = new ProgWin();
        	openFile(pw, false);
        }
        else if(o.equals(exportComm))
        {
        	exportCommunity(graph.getGraph());
        }
        else if(o.equals(exportAllComm))
        {
        	exportAllCommunities();
        }
        else if(o.equals(extractAllComm))
        {
        	extractAllCommunities();
        }
        else if(o.equals(screenshot))
        {
        	takeScreenshot(graph.getVV());
        }
        else if(o.equals(exit))
        {
        	System.exit(0);
        }
        else if(o.equals(vertexComm))
        {
        	if(graph.getPickedVertices().size() > 0)
        	{
        		vList.setJList(graph.findCommunities(graph.getPickedVertices()));
        		JPanel jp = new JPanel(new BorderLayout());
        		jp.add(sortButton, BorderLayout.NORTH);
        		jp.add(vList.getJList(), BorderLayout.CENTER);
        		commPanel.setViewportView(jp);
        	}
        }
        else if(o.equals(clearSelection))
        {
        	clearGraphSelection();
        	clearListSelection();
        	clearBicliqueList();
        }
        else if(o.equals(scaling))
        {
        	graph.setScaling(scaling.getState());
        	this.update(this.getGraphics());
        }
        else if(o.equals(nodeSearch))
        {
        	String searchString = JOptionPane.showInputDialog(this, "Enter name of node:",
					"Search for node", JOptionPane.QUESTION_MESSAGE);
        	
        	Vector<VertexObj> v = searchForNode(searchString);
        	
        	if(v.size() > 0)
        	{
        		new SearchResultsWin(v, this);
        	}
        	else
        	{
        		JOptionPane.showMessageDialog(this, "The node could not be found",
    					"None found", JOptionPane.INFORMATION_MESSAGE);
        	}
        }
        else if(o.equals(commSearch))
        {
        	String searchString = JOptionPane.showInputDialog(this, "Enter ID of community:",
					"Search for node", JOptionPane.QUESTION_MESSAGE);
        	
        	Vector<Community> v = searchForCommunity(searchString);
        	
        	if(v.size() > 0)
        	{
        		new SearchResultsCommWin(v, this);
        	}
        	else
        	{
        		JOptionPane.showMessageDialog(this, "The node could not be found",
    					"None found", JOptionPane.INFORMATION_MESSAGE);
        	}
        }
        else if(o.equals(communityNetwork))
        {
        	treeMenu.createCommunityNetwork();
        }
        else if(o.equals(biHistogram))
        {
        	new HistogramBiWin();
        }
        else if(o.equals(degHistogram))
        {
        	new HistogramDegWin(treeMenu.getRoot());
        }
        else if(o.equals(commHistogram))
        {
        	new HistogramCommWin(treeMenu.getRoot());
        }
        else if(o.equals(sortButton))
        {
        	boolean typeASorting = treeMenu.changeSorting();
        	
        	if(!typeASorting)
        	{
        		sortButton.setText("Sort by A");
        	}
        	else
        	{
        		sortButton.setText("Sort by B");
        	}
        }
        else if(o.equals(help))
        {
        	BareBonesBrowserLaunch.openURL("http://www2.imm.dtu.dk/~mhs/bcfinder/documentation.html");
        }
        else if(o.equals(about))
        {
        	BareBonesBrowserLaunch.openURL("http://www2.imm.dtu.dk/~mhs/bcfinder/preview.html");
        }
    }
	
	public void stateChanged(ChangeEvent e)
	{
		Object o = e.getSource();
		if(o.equals(sizeSpinner))
		{
			SpinnerNumberModel model = (SpinnerNumberModel)sizeSpinner.getModel();
			graph.setScaleSize(model.getNumber().intValue());
			update(getGraphics());
		}
		else if(o.equals(labelSizeSpinner))
		{
			SpinnerNumberModel model = (SpinnerNumberModel)labelSizeSpinner.getModel();
			graph.setLabelSize(model.getNumber().intValue());
			update(getGraphics());
		}
	}
}
