package bicliqueviz;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;

public class CommandTree 
{
	private String sizeFile;
	private String bicliqueFile;
	private Vector<Biclique> bicliques;
	private Vector<TreeSet<Community>> communitiesTree;
	private Vector<Vector<Biclique>> bicliquesTree;
	private int maxASize = 0;
	private int maxBSize = 0;
	private final int MAX_THRESHOLD = 20;
	private final String COMMS_TEMP = "comms.tmp";
	private int minThresholdA;
	private int minThresholdB;
	private int maxThresholdA;
	private int maxThresholdB;
	private int numOfCliques = 0;
	private int numOfCliquesExcl = 0;
	private boolean isTypeASorting;
	private Vector<String> children;
	
	public CommandTree(String sz, String bi, int minThresA, int minThresB,
			int maxThresA, int maxThresB)
	{
		sizeFile = sz;
		bicliqueFile = bi;
		bicliques = new Vector<Biclique>();
		bicliquesTree = new Vector<Vector<Biclique>>();
		communitiesTree = new Vector<TreeSet<Community>>();
		children = new Vector<String>();
		minThresholdA = minThresA;
		minThresholdB = minThresB;
		maxThresholdA = maxThresA;
		maxThresholdB = maxThresB;
		
		readBicliqueFile();
		isTypeASorting = true;
	}
	
	private void readBicliqueFile()
	{
		System.out.println("Reading biclique file...");
		bicliques = new Vector<Biclique>();
		System.out.println("0%");
		findMaxSizes();		
		
		try 
		{
			File ff = new File(bicliqueFile);
			double fileSize = ff.length();
			double bytesLeft = fileSize;
			BufferedReader reader = new BufferedReader(new FileReader(bicliqueFile));
			
			String line = "";
			
			//read lines from biclique file
			//format:
			//1st line: IDs of upper part (type A) of biclique
			//2nd line: IDs of lower part (type B) of biclique
			//3rd line: empty
			//repeat until end of file
			while(line != null)
			{
				line = reader.readLine();
				if(line == null)
				{
					break;
				}
				bytesLeft -= line.getBytes().length+2;
				String[] upper = line.split("\\s");
				line = reader.readLine();
				bytesLeft -= line.getBytes().length+2;
				String[] lower = line.split("\\s");
				String sizeString = "k="+ upper.length +","+ lower.length;
				int[] bicliqueUpper = new int[upper.length];
				int[] bicliqueLower = new int[lower.length];
				
				//check if the biclique is with the specified thresholds
				//if yes, add to bicliques vector
				if(upper.length >= minThresholdA && lower.length >= minThresholdB &&
						upper.length <= maxThresholdA && lower.length <= maxThresholdB)
				{
					for(int i = 0; i < upper.length; i++)
					{
						bicliqueUpper[i] = Integer.parseInt(upper[i]);
					}
					
					for(int i = 0; i < lower.length; i++)
					{
						bicliqueLower[i] = Integer.parseInt(lower[i]);
					}
					
					int id = bicliques.size();
					Biclique bn = new Biclique(id, bicliqueUpper, bicliqueLower, sizeString);
					bicliques.add(bn);
				}

				line = reader.readLine();
				bytesLeft -= line.getBytes().length+2;
				
				//update progress bar
				if(bytesLeft > 0)
				{
					double progress = 100.0*(fileSize-bytesLeft)/fileSize;
					int p = Math.min((int)progress, 100);
					System.out.println(p+"%");
				}
				else
				{
					System.out.println("100%");
				}
				
				
			}
			reader.close();
			
			
		} 
		catch (FileNotFoundException e) 
		{
			e.printStackTrace();
		}
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * @return all communities in tree format
	 */
	public Vector<TreeSet<Community>> getCommunitiesTree()
	{
		return communitiesTree;
	}
	
	/**
	 * @return all bicliques
	 */
	public Vector<Biclique> getBicliques()
	{
		return bicliques;
	}
	
	public Vector<String> getChildren()
	{
		return children;
	}
	
	public void createTree(InfoFile info)
	{
		System.out.println("Creating tree menu...");
		System.out.println("0%");
		File tmpFile = new File(COMMS_TEMP);
		double fileSize = tmpFile.length();
		double bytesLeft = fileSize;
		int id = 0;
		
		try 
		{
			BufferedReader reader = new BufferedReader(new FileReader(COMMS_TEMP));
			
			String line = "";
			
			//read lines of temporary community file
			while(line != null)
			{				
				Vector<Biclique> bbcs = new Vector<Biclique>();
				TreeSet<Community> ccs = new TreeSet<Community>(new CommunityComparer());
				
				line = reader.readLine();
				
				if(line == null)
				{
					break;
				}
				bytesLeft -= line.getBytes().length+2;
				
				line = line.replace("(", "");
				line = line.replace(")", "");
				String[] pair = line.split(",");
				NumberPair np = new NumberPair(Integer.parseInt(pair[0]), Integer.parseInt(pair[1]));
				CommunityTreeNode ctn = new CommunityTreeNode(np.getA(),np.getB());
				children.add(ctn.toString());
				
				line = reader.readLine();
				bytesLeft -= line.getBytes().length+2;
				int numOfComms = Integer.parseInt(line);
				
				//go through all communities of type k=a,b
				for(int i = 0; i < numOfComms; i++)
				{
					Vector<Biclique> bcs = new Vector<Biclique>();
					
					line = reader.readLine();
					bytesLeft -= line.getBytes().length+2;
					String[] splitLine = line.split("\\s");
					
					for(int j = 0; j < splitLine.length; j++)
					{
						Biclique b = bicliques.get(Integer.parseInt(splitLine[j]));
						bcs.add(b);
						bbcs.add(b);
					}
					
					Community c = new Community(bcs, id, ctn.toString());
					id++;
					c.incrementVertexOccurs(info);
					ccs.add(c);
				}
				line = reader.readLine();
				bytesLeft -= line.getBytes().length+2;

				bicliquesTree.add(bbcs);
				communitiesTree.add(ccs);
				if(bytesLeft > 0)
				{
					double progress = 100.0*(fileSize-bytesLeft)/fileSize;
					int p = Math.min((int)progress, 100);
					System.out.println(p+"%");
				}
				else
				{
					System.out.println("100%");
				}
			}
			reader.close();
			
			
			tmpFile.delete();
		} 
		catch (FileNotFoundException e) 
		{
			System.err.println("Error creating menu: "+ COMMS_TEMP +" missing");
			System.exit(1);
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
	
	private void findMaxSizes()
	{
		try 
		{
			BufferedReader reader = new BufferedReader(new FileReader(sizeFile));
			
			String line = reader.readLine();
			
			while(line != null)
			{
				String[] split = line.split("\\s");
				maxASize = Math.min(Math.max(Integer.parseInt(split[0]), maxASize), MAX_THRESHOLD);
				maxBSize = Math.min(Math.max(Integer.parseInt(split[1]), maxBSize), MAX_THRESHOLD);
				
				if(!(Integer.parseInt(split[0]) == 1 && Integer.parseInt(split[1]) == 1))
				{
					numOfCliquesExcl++;
				}
				numOfCliques++;
				line = reader.readLine();
			}
		} 
		catch (FileNotFoundException e) 
		{
			e.printStackTrace();
		}
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
	
	public void createOverlapMatrix2()
	{
		long t1 = System.currentTimeMillis();
		System.out.println("Creating overlap matrix...");
		System.out.println("0%");
		
		//Eligible diagonals
		NumberPair[] diagonals = new NumberPair[bicliques.size()];
		
		for(int i = 0; i < diagonals.length; i++)
		{
			diagonals[i] = null;
		}
		
		//Matrix m[a][b] = vector of all overlapping bicliques and
		//that pass the (a+1,b+1) threshold
		Vector<NumberPair>[][] abMatrix = new Vector[maxASize][maxBSize];
		
		for(int i = 0; i < maxASize; i++)
		{
			for(int j = 0; j < maxBSize; j++)
			{
				abMatrix[i][j] = null;
			}
		}
		
		for(int i = 0; i < bicliques.size(); i++)
		{
			int[] upper = bicliques.get(i).getBicliqueUpper();
			int[] lower = bicliques.get(i).getBicliqueLower();
			
			int j = i;
			HashSet<Integer> currentUpperSet = new HashSet<Integer>();
			HashSet<Integer> currentLowerSet = new HashSet<Integer>();
			
			for(int k = 0; k < upper.length; k++)
			{
				currentUpperSet.add(new Integer(upper[k]));
			}
			for(int k = 0; k < lower.length; k++)
			{
				currentLowerSet.add(new Integer(lower[k]));
			}
			
			//check diagonal to make sure we only include bicliques of correct size
			if(upper.length >= minThresholdA && lower.length >= minThresholdB)
			{
				if(diagonals[i] == null)
				{
					diagonals[i] = new NumberPair(upper.length,lower.length);
				}
			}
			
			j++;
			
			while(j < bicliques.size())
			{
				int[] upper2 = bicliques.get(j).getBicliqueUpper();
				int[] lower2 = bicliques.get(j).getBicliqueLower();
				
				if(upper2.length >= minThresholdA && lower2.length >= minThresholdB)
				{
					if(diagonals[j] == null)
					{
						diagonals[j] = new NumberPair(upper2.length,lower2.length);
					}
				}
				
				int upperCount = 0;
				int lowerCount = 0;
				
				//Number of overlaps in upper part
				for(int k = 0; k < upper2.length; k++)
				{
					if(currentUpperSet.contains(new Integer(upper2[k])))
					{
						upperCount++;
					}
				}
				
				//Number of overlaps in lower part
				for(int k = 0; k < lower2.length; k++)
				{
					if(currentLowerSet.contains(new Integer(lower2[k])))
					{
						lowerCount++;
					}
				}
				
				//Remaining off diagonal bicliques must be great than or equal to a-1,b-1
				if(upperCount >= minThresholdA-1 && lowerCount >= minThresholdB-1)
				{
					if(upperCount > 0 && lowerCount > 0 ||
							upperCount > 0 && lowerCount == 0 ||
							upperCount == 0 && lowerCount > 0)
					{
						NumberPair ijPair = new NumberPair(upperCount,lowerCount);
						NumberPair jjPair = new NumberPair(upper2.length,lower2.length);
						
						if(diagonals[j] != null)
						{
							int aMax = Math.min(ijPair.getA()+1, jjPair.getA());
							int bMax = Math.min(ijPair.getB()+1, jjPair.getB());
							
							aMax = Math.min(aMax, MAX_THRESHOLD);
							bMax = Math.min(bMax, MAX_THRESHOLD);
							
							int maxA = 0;
							int maxB = 0;
							for(int a = 1; a <= aMax; a++)
							{
								for(int b = 1; b <= bMax; b++)
								{
									if(a == 1 && b == 1)
									{
										b = 2;
									}
									
									if(upper.length >= a && lower.length >= b &&
											upper2.length >= a && lower2.length >= b &&
											upperCount >= a-1 && lowerCount >= b-1)
									{
										if(abMatrix[a-1][b-1] == null)
										{
											abMatrix[a-1][b-1] = new Vector<NumberPair>();
										}
	
										abMatrix[a-1][b-1].add(new NumberPair(i,j));
										
										maxA = Math.max(a-1, maxA);
										maxB = Math.max(b-1, maxB);
									}
								}
								
								System.out.println("("+i+","+j+") "+upper.length+"\t"+lower.length+"\t"+upper2.length+"\t"+lower2.length+"\t"+maxA+"\t"+maxB);
							}
							System.out.println("final ("+i+","+j+") "+upper.length+"\t"+lower.length+"\t"+upper2.length+"\t"+lower2.length+"\t"+upperCount+"\t"+lowerCount+"\n");
						}
					}
				}
				
				j++;
			}
			double k = i;
			double l = bicliques.size();
			double progress = 100.0*k/l;
			int p = Math.min((int)progress, 100);
			System.out.println(p+"%");
		}
		
		long t2 = System.currentTimeMillis();
		System.out.println("Overlap: "+(t2-t1));
		try 
		{
			BufferedWriter writer = new BufferedWriter(new FileWriter("times.txt", true));
			writer.write("Overlap: "+(t2-t1)+"\n");
			writer.close();
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}

		diagonals = null;
		
		System.out.println("Creating communities...");
		System.out.println("0%");
		
		double kk = 0;
		double l = maxASize*maxBSize;
		
		System.out.println("first done");
		
//		method 2
		for(int a = 0; a < maxASize; a++)
		{
			for(int b = 0; b < maxBSize; b++)
			{
				long free = Runtime.getRuntime().totalMemory()-Runtime.getRuntime().freeMemory();
				System.out.println("Free Memory: "+free);
				double progress = 100.0*((double)kk)/l;
				int p = Math.min((int)progress, 100);
				System.out.println(p+"%");
				kk++;
				if(abMatrix[a][b] != null)
				{
					Vector<HashSet<Integer>> com = new Vector<HashSet<Integer>>();
					
					for(int k = 0; k < abMatrix[a][b].size(); k++)
					{
						System.out.println(k+"/"+abMatrix[a][b].size() +" ("+a+","+b+")");
						
						int i = abMatrix[a][b].get(k).getA();
						int j = abMatrix[a][b].get(k).getB();
						
						abMatrix[a][b].set(k, null);
						
						if(com.size() > 0)
						{
							boolean found = false;
							for(int c = 0; c < com.size(); c++)
							{
								if(com.get(c).contains(i) || com.get(c).contains(j))
								{
									boolean found2 = false;
									for(int d = c+1; d < com.size(); d++)
									{
										if(com.get(d).contains(i) || com.get(d).contains(j))
										{
											HashSet<Integer> set = com.remove(d);
											set.add(i);
											set.add(j);
											com.get(c).addAll(set);
											found2 = true;
											break;
										}
									}
									
									if(!found2)
									{
										com.get(c).add(i);
										com.get(c).add(j);
									}
									found = true;
									break;
								}
							}
							
							if(!found)
							{
								HashSet<Integer> set = new HashSet<Integer>();
								set.add(i);
								set.add(j);
								com.add(set);
							}
						}
						else
						{
							HashSet<Integer> set = new HashSet<Integer>();
							set.add(i);
							set.add(j);
							com.add(set);
						}
					}
					
					Vector<Collection<Integer>> cs = new Vector<Collection<Integer>>();
					
					Iterator<HashSet<Integer>> it3 = com.iterator();
					
					while(it3.hasNext())
					{
						Set<Integer> gg = it3.next();
						if(gg.size() > 1)
						{
							cs.add(gg);
						}
					}
					
					if(cs.size() > 0)
					{
						try 
						{
							BufferedWriter writer = new BufferedWriter(new FileWriter(COMMS_TEMP, true));
							
							NumberPair np = new NumberPair(a+1,b+1);
							writer.write(np.toString()+"\n");
							writer.write(cs.size()+"\n");
							
							for(int i = 0; i < cs.size(); i++)
							{
								Integer[] ii = cs.get(i).toArray(new Integer[0]);
								String s = "";
								for(int j = 0; j < ii.length; j++)
								{
									s += ii[j]+"\t";
								}
								s = s.trim();
								writer.write(s+"\n");
							}
							writer.write("\n");
							writer.close();
						} 
						catch (IOException e) 
						{
							e.printStackTrace();
						}
					}
				}
				abMatrix[a][b] = null;
			}
		}
	}
}
