package bicliqueviz;

import java.awt.Color;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Hashtable;

/**
 * 
 * This class represents information given from optional ID files for the vertices of the graph
 * Maps ids of vertices to real names for vertex A and B
 * 
 * @author Martin Schwartz
 *
 */
public class InfoFile 
{
	private Hashtable<IDNodeTypePair, VertexObj> idToInfo = new Hashtable<IDNodeTypePair, VertexObj>();
	private ProgWin pw;

	/**
	 * @param ida name of info file (type A)
	 * @param idb name of info file (type B)
	 * @param p pointer to progress bar
	 */
	public InfoFile(String ida, String idb, ProgWin p) 
	{
		pw = p;
		if(pw != null)
		{
			pw.setTextAreaHeader("Reading info files...");
			pw.setValue(0);
		}
		else
		{
			System.out.println("Reading info files...");
			System.out.println("0%");
		}
		try 
		{
			if(ida != null)
			{
				File ff = new File(ida);
				double fileSize = ff.length();
				double bytesLeft = fileSize;
				BufferedReader reader = new BufferedReader(new FileReader(ida));
				
				String line = reader.readLine();
				
				while(line != null)
				{
					bytesLeft -= line.getBytes().length+2;
					String[] splitLine = line.split("\\s", 2);
					String[] v_args = splitLine[1].split("\t");
					
					Color c = null;
					String n = v_args[0];
					String s = null;
					if(v_args.length == 3)
					{
						String[] rgb = v_args[1].split(",");
						c = new Color(Integer.parseInt(rgb[0]),Integer.parseInt(rgb[1]),Integer.parseInt(rgb[2]));
						s = v_args[2];
					}
					if(v_args.length == 2)
					{
						if(v_args[1].contains(","))
						{
							String[] rgb = v_args[1].split(",");
							c = new Color(Integer.parseInt(rgb[0]),Integer.parseInt(rgb[1]),Integer.parseInt(rgb[2]));
						}
						else
						{
							s = v_args[1];
						}
					}
					
					Integer id = Integer.parseInt(splitLine[0]);
					IDNodeTypePair idp = new IDNodeTypePair(id, true);
					
					VertexObj v = new VertexObj(n, id, true, c, s);
					idToInfo.put(idp, v);
					line = reader.readLine();
					

					if(bytesLeft > 0)
					{
						double progress = 50.0*(fileSize-bytesLeft)/fileSize;
						int pp = Math.min((int)progress, 50);
						if(pw != null)
						{
							pw.setValue(pp);
						}
						else
						{
							System.out.println(pp+"%");
						}
					}
					else
					{
						if(pw != null)
						{
							pw.setValue(100);
						}
						else
						{
							System.out.println("50%");
						}
					}
				}
				reader.close();
			}
			if(idb != null)
			{
				File ff = new File(idb);
				double fileSize = ff.length();
				double bytesLeft = fileSize;
				BufferedReader reader = new BufferedReader(new FileReader(idb));
				
				String line = reader.readLine();
				
				while(line != null)
				{
					bytesLeft -= line.getBytes().length+2;
					String[] splitLine = line.split("\\s", 2);
					String[] v_args = splitLine[1].split("\t");
					
					Color c = null;
					String s = null;
					String n = v_args[0];
					if(v_args.length == 3)
					{
						String[] rgb = v_args[1].split(",");
						c = new Color(Integer.parseInt(rgb[0]),Integer.parseInt(rgb[1]),Integer.parseInt(rgb[2]));
						s = v_args[2];
					}
					if(v_args.length == 2)
					{
						if(v_args[1].contains(","))
						{
							String[] rgb = v_args[1].split(",");
							c = new Color(Integer.parseInt(rgb[0]),Integer.parseInt(rgb[1]),Integer.parseInt(rgb[2]));
						}
						else
						{
							s = v_args[1];
						}
					}
					Integer id = Integer.parseInt(splitLine[0]);
					IDNodeTypePair idp = new IDNodeTypePair(id, false);
					
					VertexObj v = new VertexObj(n, id, false, c, s);
					idToInfo.put(idp, v);
					line = reader.readLine();
					
					if(bytesLeft > 0)
					{
						double progress = 50.0*(fileSize-bytesLeft)/fileSize+50.0;
						int pp = Math.min((int)progress, 100);
						if(pw != null)
						{
							pw.setValue(pp);
						}
						else
						{
							System.out.println(pp+"%");
						}
					}
					else
					{
						if(pw != null)
						{
							pw.setValue(100);
						}
						else
						{
							System.out.println("100%");
						}
					}
				}
				reader.close();
			}
		} 
		catch (FileNotFoundException e) 
		{
			e.printStackTrace();
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
	
	public InfoFile()
	{
		
	}
	
	/**
	 * 
	 * Set occurs for a node id
	 * 
	 * @param n pointer to node type pair
	 */
	public void setNodeIDToOccurs(IDNodeTypePair n)
	{
		if(idToInfo.get(n) != null)
		{
			idToInfo.get(n).setCliqueOccurs(idToInfo.get(n).getNumOfCliqueOccurs()+1);
		}
	}
	
	/**
	 * @return the table with id info
	 */
	public Hashtable<IDNodeTypePair, VertexObj> getIDToInfo()
	{
		return idToInfo;
	}

}
