package bicliqueviz;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Arrays;
import java.util.Vector;
import javax.swing.JOptionPane;

/**
 * 
 * This class is used to run the LCM program using a system call.
 * At the moment the system call only works from windows.
 * The lcm.exe file must be in the same directory of application itself
 * 
 * @author Martin Schwartz
 *
 */
public class LCM 
{	
	/**
	 * @param inputFile path to input file (bigraph file)
	 * @param outputFile path to output file (maximal biclique file)
	 * @param sizeFile path to size file (sizes of the bicliques)
	 */
	public static void runLCM(ProgWin pw, String inputFile, String outputFile, String sizeFile, int minThresA, int minThresB, 
			int maxThresA, int maxThresB, boolean del)
	{
		String inputTemp = inputFile+".temp";
		String outputTemp = outputFile+".temp";
		
		
		boolean correctFormat = checkFileFormat(pw, inputFile);
		
		
		if(correctFormat)
		{
			if(pw != null)
			{
				pw.setTextAreaHeader("Running LCM...");
				pw.setValue(0);
			}
			else
			{
				System.out.println("Running LCM...");
				System.out.println("0%");
			}
			
			Vector<Integer> lid = convertToLCMFormat(inputFile, inputTemp);
			long t1 = System.currentTimeMillis();
			executeLCM(inputTemp, outputTemp);
			long t2 = System.currentTimeMillis();
			System.out.println("LCM: "+(t2-t1));
			try 
			{
				BufferedWriter writer = new BufferedWriter(new FileWriter("times.txt"));
				writer.write("LCM: "+(t2-t1)+"\n");
				writer.close();
			} 
			catch (IOException e) 
			{
				e.printStackTrace();
			}
			File tempFile = new File(outputTemp);
			if(!tempFile.exists())
			{
				JOptionPane.showMessageDialog(null, "LCM output doesn't exist. If you run " +
						"Window please check that you have Cygwin installed or else " +
						"compile LCM with your own compiler.", "LCM error", JOptionPane.ERROR_MESSAGE);
				System.exit(1);
			}
			convertToFinalOutput(lid, outputTemp, outputFile, sizeFile, minThresA, minThresB, maxThresA, maxThresB);
			if(pw != null)
			{
				pw.setValue(100);
			}
			else
			{
				System.out.println("100%");
			}
		}
		else
		{
			System.exit(1);
		}
		
		if(del)
		{
			File tempFile = new File(inputTemp);
			tempFile.delete();
			tempFile = new File(outputTemp);
			tempFile.delete();
		}
	}
	
	private static boolean checkFileFormat(ProgWin pw, String s)
	{
		if(pw != null)
		{
			pw.setTextAreaHeader("Checking file format...");
			pw.setValue(0);
		}
		else
		{
			System.out.println("Checking file format...");
			System.out.println("0%");
		}
		
		try 
		{
			File ff = new File(s);
			double fileSize = ff.length();
			double bytesLeft = fileSize;
			BufferedReader reader = new BufferedReader(new FileReader(s));
			
			int lineNumber = 1;
			String line = reader.readLine();
			bytesLeft -= line.getBytes().length+2;
			
			while(line != null)
			{
				String[] splitLine = line.split("\\s");
				
				if(splitLine.length != 2)
				{
					JOptionPane.showMessageDialog(null, "The file format is invalid. The file should only contain lines with the" +
							"following format:\n<node A ID><whitespace><node B ID>", "File format error", JOptionPane.ERROR_MESSAGE);
					return false;
				}
				
				try 
				{
					Integer.parseInt(splitLine[0]);
				} 
				catch (NumberFormatException e) 
				{
					JOptionPane.showMessageDialog(null, "Line "+lineNumber+" first ID: all IDs must be " +
							"numeric and equal to 0 or greater", "Number format error", JOptionPane.ERROR_MESSAGE);
					return false;
				}
				
				if(Integer.parseInt(splitLine[0]) < 0)
				{
					JOptionPane.showMessageDialog(null, "Line "+lineNumber+" first ID: all IDs must be " +
							"equal to 0 or greater", "Number format error", JOptionPane.ERROR_MESSAGE);
					return false;
				}
				
				try 
				{
					Integer.parseInt(splitLine[1]);
				} 
				catch (NumberFormatException e) 
				{
					JOptionPane.showMessageDialog(null, "Line "+lineNumber+" second ID: all IDs must be " +
							"numeric and equal to 0 or greater", "Number format error", JOptionPane.ERROR_MESSAGE);
					return false;
				}
				
				if(Integer.parseInt(splitLine[1]) < 0)
				{
					JOptionPane.showMessageDialog(null, "Line "+lineNumber+" second ID: all IDs must be " +
							"equal to 0 or greater", "Number format error", JOptionPane.ERROR_MESSAGE);
					return false;
				}
				
				line = reader.readLine();
				if(line != null)
				{
					bytesLeft -= line.getBytes().length+2;
				}
				
				if(bytesLeft > 0)
				{
					double progress = 100.0*(fileSize-bytesLeft)/fileSize;
					int p = Math.min((int)progress, 100);
					if(pw != null)
					{
						pw.setValue(p);
					}
					else
					{
						System.out.println(p+"%");
					}
				}
				else
				{
					if(pw != null)
					{
						pw.setValue(100);
					}
					else
					{
						System.out.println("100%");
					}
				}
				lineNumber++;
			}
		} 
		catch (FileNotFoundException e) 
		{
			e.printStackTrace();
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
		
		return true;
	}
	
	private static void executeLCM(String inputFile, String outputFile)
	{	
		String osName = System.getProperty("os.name");
		
		try 
		{
			if (osName.startsWith("Mac OS") || osName.startsWith("SunOS")) 
			{ 
				File lcmFile = new File("lcm");
				if(!lcmFile.exists())
				{
					JOptionPane.showMessageDialog(null, "LCM file doesn't exist", "LCM error", JOptionPane.ERROR_MESSAGE);
					System.err.println("LCM file doesn't exist");
					System.exit(1);
				}
				Runtime rt = Runtime.getRuntime();
	            Process proc = rt.exec("./lcm CI "+ inputFile +" 1 "+ outputFile);

	            StreamGobbler s1 = new StreamGobbler ("stdin", proc.getInputStream ());
	            StreamGobbler s2 = new StreamGobbler ("stderr", proc.getErrorStream ());
	            s1.start ();
	            s2.start ();
	            proc.waitFor();
			} 
			else if (osName.startsWith("Windows")) 
			{
				File lcmFile = new File("lcm.exe");
				if(!lcmFile.exists())
				{
					JOptionPane.showMessageDialog(null, "LCM file doesn't exist", "LCM error", JOptionPane.ERROR_MESSAGE);
					System.err.println("LCM file doesn't exist");
					System.exit(1);
				}
				Runtime rt = Runtime.getRuntime();
				//JOptionPane.showMessageDialog(null, "lcm CI \'"+ inputFile +"\' 1 \'"+ outputFile+"\'", "LCM error", JOptionPane.ERROR_MESSAGE);
	            Process proc = rt.exec("lcm CI \'"+ inputFile +"\' 1 \'"+ outputFile+"\'");

	            StreamGobbler s1 = new StreamGobbler ("stdin", proc.getInputStream ());
	            StreamGobbler s2 = new StreamGobbler ("stderr", proc.getErrorStream ());
	            s1.start ();
	            s2.start ();
	            proc.waitFor();
			}
			else
			{
				File lcmFile = new File("lcm");
				if(!lcmFile.exists())
				{
					JOptionPane.showMessageDialog(null, "LCM file doesn't exist", "LCM error", JOptionPane.ERROR_MESSAGE);
					System.err.println("LCM file doesn't exist");
					System.exit(1);
				}
				Runtime rt = Runtime.getRuntime();
	            Process proc = rt.exec("./lcm CI "+ inputFile +" 1 "+ outputFile);

	            StreamGobbler s1 = new StreamGobbler ("stdin", proc.getInputStream ());
	            StreamGobbler s2 = new StreamGobbler ("stderr", proc.getErrorStream ());
	            s1.start ();
	            s2.start ();
	            proc.waitFor();
			}
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
		catch (InterruptedException e) 
		{
			e.printStackTrace();
		}
	}
	
	private static Vector<Integer> convertToLCMFormat(String inputFile, String outputFile)
	{
		Vector<Integer> lineToID = new Vector<Integer>();
		Vector<Vector<Integer>> lines = new Vector<Vector<Integer>>();
		
		try 
		{
			BufferedReader reader = new BufferedReader(new FileReader(inputFile));
			BufferedWriter writer = new BufferedWriter(new FileWriter(outputFile));
			
			String line = reader.readLine();
			String[] splitLine = line.split("\\s");
			int id = Integer.parseInt(splitLine[0]);
			lines.add(new Vector<Integer>());
			lineToID.add(new Integer(id));
			
			while(line != null)
			{
				splitLine = line.split("\\s");
				id = Integer.parseInt(splitLine[0]);
				
				if(lineToID.contains(new Integer(id)))
				{
					lines.elementAt(lineToID.indexOf(new Integer(id))).add(new Integer(Integer.parseInt(splitLine[1])));
				}
				else
				{
					lineToID.add(new Integer(id));
					lines.add(new Vector<Integer>());
					lines.elementAt(lineToID.indexOf(new Integer(id))).add(new Integer(Integer.parseInt(splitLine[1])));
				}
				line = reader.readLine();
			}
			reader.close();
			
			for(int i = 0; i < lines.size(); i++)
			{
				for(int j = 0; j < lines.elementAt(i).size(); j++)
				{
					writer.write(lines.elementAt(i).elementAt(j)+" ");
				}
				writer.write("\n");
			}
			
			writer.close();
		} 
		catch (FileNotFoundException e) 
		{
			e.printStackTrace();
		}
		catch (IOException e) 
		{
			e.printStackTrace();
		}
		return lineToID;
	}
	
	private static void convertToFinalOutput(Vector<Integer> lid, 
			String inputFile, String outputFile, String sizeFile, int minThresA, int minThresB, int maxThresA, int maxThresB)
	{
		try 
		{
			BufferedReader reader = new BufferedReader(new FileReader(inputFile));
			BufferedWriter writer = new BufferedWriter(new FileWriter(outputFile));
			BufferedWriter sizeWriter = new BufferedWriter(new FileWriter(sizeFile));
			
			String line = "";
			
			while(line != null)
			{
				line = reader.readLine();
				if(line == null)
				{
					break;
				}
				if(line.equals(""))
				{
					break;
				}
				line = line.trim();
				String[] lower = line.split("\\s");
				Integer[] lowerNumLine = new Integer[lower.length];
				
				for(int i = 0; i < lowerNumLine.length; i++)
				{
					lowerNumLine[i] = Integer.valueOf(lower[i]);
				}
				Arrays.sort(lowerNumLine, new IntComparer());
				
				line = reader.readLine();
				line = line.trim();
				String[] upper = line.split("\\s");
				Integer[] upperNumLine = new Integer[upper.length];
				
				for(int i = 0; i < upperNumLine.length; i++)
				{
					upperNumLine[i] = Integer.valueOf(upper[i]);
					upperNumLine[i] = lid.get(upperNumLine[i]);
				}
				Arrays.sort(upperNumLine, new IntComparer());
				
				if(upperNumLine.length >= minThresA && lowerNumLine.length >= minThresB &&
						upperNumLine.length <= maxThresA && lowerNumLine.length <= maxThresB)
				{
					for(int i = 0; i < upperNumLine.length; i++)
					{
						writer.write(upperNumLine[i]+" ");
					}
					writer.write("\n");
					for(int i = 0; i < lowerNumLine.length; i++)
					{
						writer.write(lowerNumLine[i]+" ");
					}
					
					sizeWriter.write(upperNumLine.length +" "+ lowerNumLine.length +"\n");
					
					writer.write("\n");
					writer.write("\n");
				}
			}
			reader.close();
			writer.close();
			sizeWriter.close();
		} 
		catch (FileNotFoundException e) 
		{
			e.printStackTrace();
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
}
