package bicliqueviz;

import java.awt.Color;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JList;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.ListSelectionModel;

/**
 * 
 * Class for vertex community list
 * 
 * @author the1schwartz
 *
 */
public class VertexCommList implements ListSelectionListener
{
	private JList vList;
	private BiGraph graph;
	private BiCliqueWin win;
	private TabbedPane upperTabs;
	private TabbedPane lowerTabs;
	private final String TITLE = "Comm. of selected N";
	private String lastTabTitle;

	public VertexCommList(BiGraph bg, BiCliqueWin w, TabbedPane t1, 
			TabbedPane t2) 
	{
		win = w;
		graph = bg;
		upperTabs = t1;
		lowerTabs = t2;
		vList = new JList();
		vList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		vList.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK), TITLE, TitledBorder.LEFT, TitledBorder.TOP));
		vList.addListSelectionListener(this);
		lastTabTitle = null;
	}
	
	/**
	 * @return the JList
	 */
	public JList getJList()
	{
		return vList;
	}
	
	/**
	 * clears the list
	 */
	public void clearList()
	{
		Vector<String> indices = new Vector<String>();
		for(int i = 0; i < upperTabs.getTabbedPane().getTabCount(); i++)
		{
			String t = upperTabs.getTabbedPane().getTitleAt(i);
			
			if(t.contains("community"))
			{
				indices.add(t);
			}
		}
		
		for(int i = 0; i < indices.size(); i++)
		{
			upperTabs.removeTab(indices.get(i));
			lowerTabs.removeTab(indices.get(i));
		}
		
		setJList(new JList());
		win.setCommPanel(vList);
		
		lastTabTitle = null;
	}
	
	/**
	 * @param jl the JList be set as the new list
	 */
	public void setJList(JList jl)
	{
		vList = jl;
		vList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		vList.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK), TITLE, TitledBorder.LEFT, TitledBorder.TOP));
		vList.addListSelectionListener(this);
	}
	
	public void valueChanged(ListSelectionEvent e) 
	{
		Community c = (Community)vList.getSelectedValue();
		
		if(c != null && e.getValueIsAdjusting())
		{
			for(int i = 0; i < upperTabs.getTabbedPane().getTabCount(); i++)
			{
				upperTabs.getTabbedPane().remove(i);
				lowerTabs.getTabbedPane().remove(i);
			}
			if(win.getTreeMenu().getJTree().getSelectionCount() > 0)
			{
				JList tmp = vList;
				win.getTreeMenu().getJTree().clearSelection();
				setJList(tmp);
				win.setCommPanel(vList);
			}	
			graph.changeGraph(graph.createBiclique(c.getBicliques()));
			graph.getVV().setVisible(true);
			
			if(upperTabs.getTabbedPane().indexOfTab(c.toString()) == -1)
			{
				if(lastTabTitle != null)
				{
					upperTabs.removeTab(lastTabTitle);
					lowerTabs.removeTab(lastTabTitle);
				}
				Collection<VertexObj> nodes = graph.getGraph().getVertices();
				Iterator<VertexObj> it = nodes.iterator();
				Vector<VertexObj> vas = new Vector<VertexObj>();
				Vector<VertexObj> vbs = new Vector<VertexObj>();
				while(it.hasNext())
				{
					VertexObj v = it.next();
					
					if(v.isVertexA())
					{
						vas.add(v);
					}
					else
					{
						vbs.add(v);
					}
				}
				
				graph.setVertexLocations(vas.size(), vbs.size());
				
				JList verticesA = new JList(vas);
				JList verticesB = new JList(vbs);
				
				verticesA.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK), "Vertices", TitledBorder.CENTER, TitledBorder.TOP));
				verticesB.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK), "Vertices", TitledBorder.CENTER, TitledBorder.TOP));
				
				Vector<EdgeObj> list = new Vector<EdgeObj>(graph.getGraph().getEdges());
				JList edgesA = new JList(list);
				JList edgesB = new JList(list);
				
				edgesA.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK), "Edges", TitledBorder.CENTER, TitledBorder.TOP));
				edgesB.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK), "Edges", TitledBorder.CENTER, TitledBorder.TOP));
				
				JList bicliquesA = new JList(c.getBicliques());
				JList bicliquesB = new JList(c.getBicliques());
				
				bicliquesA.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK), "Bicliques", TitledBorder.CENTER, TitledBorder.TOP));
				bicliquesB.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK), "Bicliques", TitledBorder.CENTER, TitledBorder.TOP));
				
				upperTabs.addTab(c.toString(), verticesA, edgesA, bicliquesA, graph);
				lowerTabs.addTab(c.toString(), verticesB, edgesB, bicliquesB, graph);
				
				lastTabTitle = c.toString();
			}
		}
	}
}
